'*****************************************************************************************************************
'*  Name    : I2CPrintHardCommand.inc                                                                            *
'*  Purpose : allow Proton Print commands to access the standard 1602 LCD via I2C backpack board                 * 
'*  Author  : George Towler                                                                                      *
'*  Notice  : I claim no copyright but maybe Barak does                                                          *
'*  Date    : 29/08/2014                                                                                         *
'*  Version : 0.4.0                                                                                              *
'*  Notes   : see http://www.protonbasic.co.uk/showthread.php/62478-%93Print%94-command-for-SPI-Alfa-numeric-LCD *                                                  *
'*          :     post #6   Barak shows how to replace Proton commands and how to deal with control (FE$) bytes  *                                          *                                                                                     *
'*****************************************************************************************************************
'History 0.4.0 - HORRIBLE kludge to hide double char error, I still have no idea why this happens. Also no idea 
'                what I've broken as a result of this "fix"! 
'        0.3.1 - Fixed first char swallowed. Double char error discovered and demonstrated in test data
'        0.3.0 - Backlight controlled by $FE Commands - $FE 03 on, $FE 04 off
'        0.2.0 - code rehashed as include file with unique names - see I2CPrintCommand_Demo.bas for usage 
'        0.1.2 - added conditional compilation of data masking to allow for backpack data on high or low nbble
'        0.1.1 - made control bits sybolic to allow for backpacks with different bit configurations
'        0.1.0 - first posted

' New layout shamelessly stolen from Proton_Reversed_Print.zip, which is an excellent example of command 
' replacement and tight code. 

  #Disable Print                              

  ;***************************************************************************************************************
  ; Configure items to suit your LCD and I2C hardware, I2CPrint_DataOnHighBits remmed if data is on low bits     *
  ;                                                                                                              *
  Symbol I2CPrint_Part_address    = $0E         ' PCF8574 Slave Address 0,1,0,0,A2,A1,A0,RW  where W=0           *
  Symbol I2CPrint_InitForLCDLines = $28         ' $28 for two lines                                              *
  Symbol I2CPrint_RS              = %00000001   '                                                                *
  Symbol I2CPrint_RW              = %00000010   '                                                                *
  Symbol I2CPrint_E               = %00000100   '                                                                *
  Symbol I2CPrint_BackLight       = %00001000   '                                                                *
  $define I2CPrint_DataOnHighBits               '                                                                *
  ;***************************************************************************************************************
  ;
  ;  Example of Backpack with control bits on the low order nibble, I2CPrint_DataOnHighBits would be remmed
  ;  Symbol I2CPrint_RS             = %01000000
  ;  Symbol I2CPrint_RW             = %00100000
  ;  Symbol I2CPrint_E              = %00010000
  ;  Symbol I2CPrint_BackLight      = %10000000
  ;  A0=D0  A1=D1  A2=D2  A3=D3  A4=E  A5=RW  A6=RS  A7=I2CPrint_BackLight
  ;
  Symbol I2CPrint_RS_E           = I2CPrint_RS | I2CPrint_E
  Symbol I2CPrint_Control        = %01000000                         
  Symbol I2CPrint_Address        = I2CPrint_Control + I2CPrint_Part_address

  $ifdef I2CPrint_DataOnHighBits
    Symbol I2CPrint_Init1          = %00110000 
    Symbol I2CPrint_Init1_E        = %00110000 | I2CPrint_E
    Symbol I2CPrint_Init2          = %00110000  
    Symbol I2CPrint_Init2_E        = %00110000 | I2CPrint_E
    Symbol I2CPrint_Init3          = %00100000 
    Symbol I2CPrint_Init3_E        = %00100000 | I2CPrint_E
  $else
    Symbol I2CPrint_Init1          = %00000011  
    Symbol I2CPrint_Init1_E        = %00000011 | I2CPrint_E
    Symbol I2CPrint_Init2          = %00000011  
    Symbol I2CPrint_Init2_E        = %00000011 | I2CPrint_E
    Symbol I2CPrint_Init3          = %00000010  
    Symbol I2CPrint_Init3_E        = %00000010 | I2CPrint_E
  $endif

  Symbol I2CPrint_tComOrData = BPF.0
  Symbol I2CPrint_tInitialised = BPF.1  

  Dim BPF As Byte System
  Dim I2CPrint_PrintData As Byte
  Dim I2CPrint_HighData As Byte
  Dim I2CPrint_LowData As Byte
  Dim I2CPrint_dummy As Byte
  Dim I2CPrint_RSFlag As Byte  
  Dim Backlight_State As Byte
  Dim I2CPrintStore As Byte
  Dim I2CPrint_PrevChar As Byte

  GoTo I2CPrintCommand_End
  @Print                                              ; assembler Print redirect
  Wreg_Byte I2CPrint_PrintData                        ; load LCDByte to W Register


  If I2CPrint_tInitialised = 0 Then                   ; Has LCD been inititalised?
    I2CPrintStore = I2CPrint_PrintData
    HBusOut I2CPrint_Address, [I2CPrint_Init1_E]      ; E hi and send command nibble 0011 (3)    
    HBusOut I2CPrint_Address, [I2CPrint_Init1]        ; E lo     
    DelayMS 20
    HBusOut I2CPrint_Address, [I2CPrint_Init2_E]      ; E hi and send command nibble 0011 (3)    
    HBusOut I2CPrint_Address, [I2CPrint_Init2]        ; E lo     
    DelayMS 20
    HBusOut I2CPrint_Address, [I2CPrint_Init3_E]      ; E hi and send command nibble 0010 (2)    
    HBusOut I2CPrint_Address, [I2CPrint_Init3]        ; E lo     
    DelayMS 20
    I2CPrint_PrintData = I2CPrint_InitForLCDLines     ; 4 bit mode and n lines 
    GoSub I2CPrint_SendControl
    I2CPrint_PrintData = $0C                          ; enable display, cursor and blink off
    GoSub I2CPrint_SendControl
    I2CPrint_PrintData = $06                          ; move cursor after each write 
    GoSub I2CPrint_SendControl                    
    I2CPrint_PrintData = $80                          ; goto column 1 line 1 
    GoSub I2CPrint_SendControl
    Set I2CPrint_tInitialised                         ; Indicate that the LCD is initialised
    I2CPrint_PrintData = I2CPrintStore
  EndIf

  If I2CPrint_PrintData = 0xFE Then                   ; it is a command header
    I2CPrint_RSFlag = 0                               ; clear a flag so we know the next byte will be a command
  Else
    If I2CPrint_RSFlag = 0 Then                       ; this is a command byte
      If I2CPrint_PrintData = 03 Then
        Backlight_State = I2CPrint_BackLight  
        ElseIf I2CPrint_PrintData = 04 Then
        Backlight_State = 0  
      Else
        GoSub I2CPrint_SendControl
      End If
    Else
      If I2CPrint_PrintData = 01 Then                 ; HORRIBLE kludge - PRINT seems to always send 01 on double char err
        I2CPrint_PrintData = I2CPrint_PrevChar
        GoSub I2CPrint_SendData
      Else
        I2CPrint_PrevChar = I2CPrint_PrintData
        GoSub I2CPrint_SendData
      EndIf
    EndIf
    I2CPrint_RSFlag = 1                               ; set the flag so the next byte will be send as Data
  EndIf                                     
  Return


' ****************************************************************************
' **                             S U B R O U T I N E S                      **
' ****************************************************************************

I2CPrint_SendData:
  $ifdef I2CPrint_DataOnHighBits
    I2CPrint_HighData = I2CPrint_PrintData & %11110000            ; mask LSBs
    I2CPrint_LowData = I2CPrint_PrintData << 4                    ; shift left
  $else
    I2CPrint_LowData = I2CPrint_PrintData & %00001111             ; mask MSBs
    I2CPrint_HighData = I2CPrint_PrintData >> 4                   ; shift Right
  $endif

  I2CPrint_dummy = I2CPrint_RS_E | I2CPrint_HighData | Backlight_State  ; data and e hi
  HBusOut I2CPrint_Address, [I2CPrint_dummy]             
  I2CPrint_dummy = I2CPrint_RS | I2CPrint_HighData | Backlight_State    ; data and e lo
  HBusOut I2CPrint_Address, [I2CPrint_dummy]              
  I2CPrint_dummy = I2CPrint_RS | Backlight_State                        ; data and e hi
  HBusOut I2CPrint_Address, [I2CPrint_dummy]                            ; Delay, RS on throughout
  I2CPrint_dummy = I2CPrint_RS_E | I2CPrint_LowData | Backlight_State   ; data and e hi
  HBusOut I2CPrint_Address, [I2CPrint_dummy]              
  I2CPrint_dummy = I2CPrint_RS | I2CPrint_LowData | Backlight_State     ; data and e lo
  HBusOut I2CPrint_Address, [I2CPrint_dummy]         
  Return
  
  
I2CPrint_SendControl:
  $ifdef I2CPrint_DataOnHighBits
    I2CPrint_HighData = I2CPrint_PrintData & %11110000                  ; mask LSBs
    I2CPrint_LowData = I2CPrint_PrintData << 4                          ; shift left
  $else
    I2CPrint_LowData = I2CPrint_PrintData & %00001111                   ; mask MSBs
    I2CPrint_HighData = I2CPrint_PrintData >> 4                         ; shift Right
  $endif

  I2CPrint_dummy = I2CPrint_E | I2CPrint_HighData | Backlight_State     ; data and e hi
  HBusOut I2CPrint_Address, [I2CPrint_dummy]              
  I2CPrint_dummy = I2CPrint_HighData | Backlight_State                  ; data and e lo
  HBusOut I2CPrint_Address, [I2CPrint_dummy]              
  I2CPrint_dummy = Backlight_State                                      ; 
  HBusOut I2CPrint_Address, [I2CPrint_dummy]                            ; Delay, Backlight on throughout
  I2CPrint_dummy = I2CPrint_E | I2CPrint_LowData | Backlight_State      ; data and e hi
  HBusOut I2CPrint_Address, [I2CPrint_dummy]              
  I2CPrint_dummy = I2CPrint_LowData | Backlight_State                   ; data and e lo
  HBusOut I2CPrint_Address, [I2CPrint_dummy]         
  DelayMS 20
  Return

I2CPrintCommand_End: